@@ -57,17 +57,19 @@
 #define DAMAGE_EVENTS_BOOL_OPTION               31
 #define CONSTANT_DPI_BOOL_OPTION                32
 #define PROBE_ALL_GPUS_BOOL_OPTION              33
-#define DYNAMIC_TWINVIEW_BOOL_OPTION            34
-#define INCLUDE_IMPLICIT_METAMODES_BOOL_OPTION  35
-#define USE_EVENTS_BOOL_OPTION                  36
-#define CONNECT_TO_ACPID_BOOL_OPTION            37
-#define ENABLE_ACPI_HOTKEYS_BOOL_OPTION         38
-#define MODE_DEBUG_BOOL_OPTION                  39
-#define THERMAL_CONFIGURATION_CHECK_BOOL_OPTION 40
-#define PRESERVE_BUSID_BOOL_OPTION              41
-#define BASE_MOSAIC_BOOL_OPTION                 42
+#define INCLUDE_IMPLICIT_METAMODES_BOOL_OPTION  34
+#define USE_EVENTS_BOOL_OPTION                  35
+#define CONNECT_TO_ACPID_BOOL_OPTION            36
+#define ENABLE_ACPI_HOTKEYS_BOOL_OPTION         37
+#define MODE_DEBUG_BOOL_OPTION                  38
+#define THERMAL_CONFIGURATION_CHECK_BOOL_OPTION 39
+#define PRESERVE_BUSID_BOOL_OPTION              40
+#define BASE_MOSAIC_BOOL_OPTION                 41
+#define ALLOW_EMPTY_INITIAL_CONFIGURATION       42
+#define DELETE_UNUSED_DP12_DPYS                 43
+#define INBAND_STEREO_SIGNALING                 44
 
-#define XCONFIG_BOOL_OPTION_COUNT (BASE_MOSAIC_BOOL_OPTION + 1)
+#define XCONFIG_BOOL_OPTION_COUNT (INBAND_STEREO_SIGNALING + 1)
 
 /* # of 32-bit variables needed to hold all the boolean options (bits) */
 #define XCONFIG_BOOL_OPTION_SLOTS  \
@@ -121,6 +123,8 @@ typedef struct __options {
     int cool_bits;
     int nvidia_3dvision_display_type;
 
+    int num_x_screens;
+
     char *xconfig;
     char *output_xconfig;
     char *layout;
@@ -178,9 +182,10 @@ typedef struct _device_rec {
     NvCfgDeviceHandle handle;
     int crtcs;
     char *name;
+    char *uuid;
     unsigned int displayDeviceMask;
     int nDisplayDevices;
-    DisplayDevicePtr displayDevices;    
+    DisplayDevicePtr displayDevices;
 } DeviceRec, *DevicePtr;
 
 typedef struct {
